# coding: utf-8
from spyne.decorator import rpc
from spyne.model.primitive import String

from . import utils
from .types import GetIndexResponse, IndexPeriod


@rpc(
    String, String, IndexPeriod,
    _in_variable_names=dict(
        report_code='IndexCode',
        param='IndexParam',
        period='IndexPeriod',
    ),
    _returns=GetIndexResponse,
    _out_message_name='GetIndexResponse',
)
def GetIndexRequest(ctx, report_code, param, period):
    from ..data import utils as data_utils
    assert isinstance(period, IndexPeriod)

    utils.validate_request(ctx, report_code, period)

    kwargs = dict(
        ctx=ctx,
        report_code=report_code,
        period_type=period.IndexPeriodType,
        param=param,
    )
    kwargs.update(
        utils.get_report_period(
            period.IndexPeriodType,
            **dict(
                period_date=utils.as_date(period.IndexPeriodDate),
                date_from=utils.as_date(period.IndexPeriodStartDate),
                date_to=utils.as_date(period.IndexPeriodEndDate),
            )
        ),
    )

    out_format = 'xml'
    file_name = '.'.join((report_code, out_format))
    content = data_utils.get_content(**kwargs)

    return GetIndexResponse.create(file_name, content)
