# coding: utf-8
from StringIO import StringIO
from base64 import b64decode, b64encode
from zipfile import ZIP_DEFLATED
from zipfile import ZipFile
import datetime
from spyne import Fault


def create_attachment(filename, content):
    u"""Возвращает base64-кодированный zip-архив."""
    in_memory_file = StringIO()
    with ZipFile(in_memory_file, 'w', ZIP_DEFLATED) as zip_file:
        zip_file.writestr(filename, content.encode('UTF-8'))
    return b64encode(in_memory_file.getvalue())


def unpack_attachment(content, filename):
    u"""Распаковывает base64-кодированный zip-архив."""
    with ZipFile(StringIO(b64decode(content)), 'r') as zip_file:
        with zip_file.open(filename, 'r') as zip_content:
            return zip_content.read()


def as_date(string):
    if not string:
        return
    date_format = '%d.%m.%Y'
    strptime = datetime.datetime.strptime
    return strptime(string, date_format).date()


PERIOD_TODAY = 'today'
PERIOD_YESTERDAY = 'yesterday'
PERIOD_DATE = 'date'
PERIOD_PERIOD = 'period'
PERIOD_TYPES = (PERIOD_TODAY, PERIOD_YESTERDAY, PERIOD_DATE, PERIOD_PERIOD)


def validate_request(ctx, report_code, period):
    u"""Проверяет входные данные."""
    error = None
    period_type = period.IndexPeriodType

    if period_type == PERIOD_DATE and not period.IndexPeriodDate:
        error = (
            u'Для периода `date` должен быть указан параметр `IndexPeriodDate`'
        )

    elif period_type == PERIOD_PERIOD:
        if not all((
            period.IndexPeriodStartDate, period.IndexPeriodEndDate
        )):
            error = (
                u'Для периода `period` должны быть указаны параметры '
                u'`IndexPeriodStartDate` и `IndexPeriodEndDate`'
            )
        elif (
            as_date(period.IndexPeriodStartDate) >
            as_date(period.IndexPeriodEndDate)
        ):
            error = (
                u'Дата начала периода позднее, чем дата окончания.'
            )

    if error is not None:
        raise Fault(faultstring=error)


def get_report_period(period_type, **kwargs):
    u"""Возвращает период дат отчета в зависимости от типа запроса."""
    assert period_type in PERIOD_TYPES, period_type

    result = {}
    if period_type == PERIOD_TODAY:
        today = datetime.date.today()
        result['date_from'] = result['date_to'] = today

    elif period_type == PERIOD_YESTERDAY:
        today = datetime.date.today()
        yesterday = today - datetime.timedelta(days=1)
        result['date_from'] = result['date_to'] = yesterday

    elif period_type == PERIOD_DATE:
        assert kwargs.get('period_date')
        result['date_from'] = result['date_to'] = kwargs['period_date']

    elif period_type == PERIOD_PERIOD:
        result['date_from'] = kwargs['date_from']
        result['date_to'] = kwargs['date_to']

    return result
